/// This file was generated on 2022-03-04T15:29:14.690377


import '../../pco.dart';

/// This class represents a PCO Calendar Tag Object
/// 
/// - Application:        calendar
/// - Id:                 tag
/// - Type:               Tag
/// - ApiVersion:         2020-04-08
/// - Is Deprecated:      false
/// - Is Collection Only: false
/// - Default Endpoint:   https://api.planningcenteronline.com/calendar/v2/tags
/// 
/// Description:
/// An organizational tag that can be applied to events.
/// 
/// Applied tags can be used to filter events on the calendar or
/// filter events for reports, iCal feeds, kiosk, and the widget.
/// 
/// Example:
/// ```json
/// {
///   "type": "Tag",
///   "id": "1",
///   "attributes": {
///     "color": "string",
///     "created_at": "2000-01-01T12:00:00Z",
///     "name": "string",
///     "position": 1.42,
///     "updated_at": "2000-01-01T12:00:00Z"
///   },
///   "relationships": {}
/// }
/// ```
/// 
/// Possible includes with parameter ?include=a,b
/// - tag_group: include associated tag_group 
///
/// Possible queries using parameters like ?where[key]=value or ?where[key][gt|lt]=value
/// - `color`: (URLParameter), query on a specific color, example: ?where[color]=string
/// - `created_at`: (URLParameter), query on a specific created_at, example: ?where[created_at]=2000-01-01T12:00:00Z
/// - `id`: (URLParameter), query on a specific id, example: ?where[id]=primary_key
/// - `name`: (URLParameter), query on a specific name, example: ?where[name]=string
/// - `position`: (URLParameter), query on a specific position, example: ?where[position]=1.42
/// - `updated_at`: (URLParameter), query on a specific updated_at, example: ?where[updated_at]=2000-01-01T12:00:00Z
/// 
/// Possible orderings with parameter ?order=
/// - `name`: (URLParameter), prefix with a hyphen (-name) to reverse the order
/// - `position`: (URLParameter), prefix with a hyphen (-position) to reverse the order
///
/// All Outbound Edges:
/// - `eventinstance-tag-event_instances`: https://api.planningcenteronline.com/calendar/v2/tags/1/event_instances
/// - `event-tag-events`: https://api.planningcenteronline.com/calendar/v2/tags/1/events
/// - `taggroup-tag-tag_group`: https://api.planningcenteronline.com/calendar/v2/tags/1/tag_group
/// 
/// All Inbound Edges:
/// - `tag-eventinstance-tags`: https://api.planningcenteronline.com/calendar/v2/event_instances/1/tags
/// - `tag-event-tags`: https://api.planningcenteronline.com/calendar/v2/events/1/tags
/// - `tag-organization-tags`: https://api.planningcenteronline.com/calendar/v2/tags
/// - `tag-taggroup-tags`: https://api.planningcenteronline.com/calendar/v2/tag_groups/1/tags
/// 
/// All Actions:
/// NONE
///
class PcoCalendarTag extends PcoResource {
  static const String kPcoApplication = 'calendar';
  static const String kTypeString = 'Tag';
  static const String kTypeId = 'tag';
  static const String kApiVersion = '2020-04-08';
  static const String kShortestEdgeId = 'tag-organization-tags';
  static const String kShortestEdgePathTemplate = 'https://api.planningcenteronline.com/calendar/v2/tags';
  static const String kDefaultPathTemplate = 'https://api.planningcenteronline.com/calendar/v2/tags';

  /// possible includes with parameter ?include=a,b
  /// - `tag_group`: include associated tag_group 
  static List<String> get canInclude => ['tag_group'];

  /// possible queries using parameters like ?where[key]=value or ?where[key][gt|lt]=value
  /// - `color`: (URLParameter), query on a specific color, example: ?where[color]=string
  /// - `created_at`: (URLParameter), query on a specific created_at, example: ?where[created_at]=2000-01-01T12:00:00Z
  /// - `id`: (URLParameter), query on a specific id, example: ?where[id]=primary_key
  /// - `name`: (URLParameter), query on a specific name, example: ?where[name]=string
  /// - `position`: (URLParameter), query on a specific position, example: ?where[position]=1.42
  /// - `updated_at`: (URLParameter), query on a specific updated_at, example: ?where[updated_at]=2000-01-01T12:00:00Z
  static List<String> get canQuery => ['color','created_at','id','name','position','updated_at'];

  /// possible orderings with parameter ?order=
  /// - `name`: (URLParameter), prefix with a hyphen (-name) to reverse the order
  /// - `position`: (URLParameter), prefix with a hyphen (-position) to reverse the order
  static List<String> get canOrderBy => ['name','position'];

  // By using overridden getters, the parent class can call the getter and will get the results from the
  // child class. This lets the parent access the static variables of the child class.

  @override
  String get shortestEdgePath => kShortestEdgePathTemplate;

  @override
  String get defaultPathTemplate => kDefaultPathTemplate;

  @override
  String get apiVersion => kApiVersion;

  // field mapping constants
  static const kId = 'id';
  static const kColor = 'color';
  static const kCreatedAt = 'created_at';
  static const kName = 'name';
  static const kPosition = 'position';
  static const kUpdatedAt = 'updated_at';


  // getters and setters
  @override
  List<String> get createAllowed => [];

  @override
  List<String> get updateAllowed => [];

  @override
  bool get canCreate => false;

  @override
  bool get canUpdate => false;

  @override
  bool get canDestroy => false;

  // getters for object attributes

  String get color => attributes[kColor] ?? '';
  String get name => attributes[kName] ?? '';
  double get position => attributes[kPosition] ?? 0;
  

  // setters for object attributes

  

  // additional setters and getters for assignable values

  



  // Class Constructors
  PcoCalendarTag() : super(kPcoApplication, kTypeString);
  PcoCalendarTag.fromJson(Map<String, dynamic> data, {List<Map<String, dynamic>> withIncludes = const []}): super.fromJson(kPcoApplication, kTypeString, data, withIncludes: withIncludes);



  // ---------------------------------
  // Inbound Edges
  // ---------------------------------



  /// Will get a collection of [PcoCalendarTag] objects (expecting many)
  /// using a path like this: `/calendar/v2/tags`
  /// 
  /// Available Query Filters:
  /// - `individual`
  static Future<PcoCollection<PcoCalendarTag>> get( {String? id, PlanningCenterApiQuery? query, bool allIncludes = false}) async {
    query ??= PlanningCenterApiQuery();
    if (allIncludes) query.include = PcoCalendarTag.canInclude;
    var url = '/calendar/v2/tags';
    if (id != null) url += '/$id';
    return PcoCollection.fromApiCall<PcoCalendarTag>(url, query: query, apiVersion:kApiVersion);
  }


  /// Will get a collection of [PcoCalendarTag] objects (expecting many)
  /// using a path like this: `/calendar/v2/event_instances/$eventInstanceId/tags`
  static Future<PcoCollection<PcoCalendarTag>> getFromEventInstance(String eventInstanceId, {String? id, PlanningCenterApiQuery? query, bool allIncludes = false}) async {
    query ??= PlanningCenterApiQuery();
    if (allIncludes) query.include = PcoCalendarTag.canInclude;
    var url = '/calendar/v2/event_instances/$eventInstanceId/tags';
    if (id != null) url += '/$id';
    return PcoCollection.fromApiCall<PcoCalendarTag>(url, query: query, apiVersion:kApiVersion);
  }


  /// Will get a collection of [PcoCalendarTag] objects (expecting many)
  /// using a path like this: `/calendar/v2/events/$eventId/tags`
  static Future<PcoCollection<PcoCalendarTag>> getFromEvent(String eventId, {String? id, PlanningCenterApiQuery? query, bool allIncludes = false}) async {
    query ??= PlanningCenterApiQuery();
    if (allIncludes) query.include = PcoCalendarTag.canInclude;
    var url = '/calendar/v2/events/$eventId/tags';
    if (id != null) url += '/$id';
    return PcoCollection.fromApiCall<PcoCalendarTag>(url, query: query, apiVersion:kApiVersion);
  }


  /// Will get a collection of [PcoCalendarTag] objects (expecting many)
  /// using a path like this: `/calendar/v2/tag_groups/$tagGroupId/tags`
  static Future<PcoCollection<PcoCalendarTag>> getFromTagGroup(String tagGroupId, {String? id, PlanningCenterApiQuery? query, bool allIncludes = false}) async {
    query ??= PlanningCenterApiQuery();
    if (allIncludes) query.include = PcoCalendarTag.canInclude;
    var url = '/calendar/v2/tag_groups/$tagGroupId/tags';
    if (id != null) url += '/$id';
    return PcoCollection.fromApiCall<PcoCalendarTag>(url, query: query, apiVersion:kApiVersion);
  }


  // --------------------------------
  // Outbound Edges
  // --------------------------------
  // Instance functions to traverse outbound edges

  /// Will get a collection of [PcoCalendarEventInstance] objects (expecting many)
  /// using a path like this: `https://api.planningcenteronline.com/calendar/v2/tags/1/event_instances`
  Future<PcoCollection<PcoCalendarEventInstance>> getEventInstances({PlanningCenterApiQuery? query, bool allIncludes = false}) async {
    query ??= PlanningCenterApiQuery();
    if (allIncludes) query.include = PcoCalendarEventInstance.canInclude;
    var url = '$apiEndpoint/event_instances';
    return PcoCollection.fromApiCall<PcoCalendarEventInstance>(url, query: query, apiVersion: apiVersion);
  }

  /// Will get a collection of [PcoCalendarEvent] objects (expecting many)
  /// using a path like this: `https://api.planningcenteronline.com/calendar/v2/tags/1/events`
  Future<PcoCollection<PcoCalendarEvent>> getEvents({PlanningCenterApiQuery? query, bool allIncludes = false}) async {
    query ??= PlanningCenterApiQuery();
    if (allIncludes) query.include = PcoCalendarEvent.canInclude;
    var url = '$apiEndpoint/events';
    return PcoCollection.fromApiCall<PcoCalendarEvent>(url, query: query, apiVersion: apiVersion);
  }

  /// Will get a collection of [PcoCalendarTagGroup] objects (expecting many)
  /// using a path like this: `https://api.planningcenteronline.com/calendar/v2/tags/1/tag_group`
  Future<PcoCollection<PcoCalendarTagGroup>> getTagGroup({PlanningCenterApiQuery? query, bool allIncludes = false}) async {
    query ??= PlanningCenterApiQuery();
    if (allIncludes) query.include = PcoCalendarTagGroup.canInclude;
    var url = '$apiEndpoint/tag_group';
    return PcoCollection.fromApiCall<PcoCalendarTagGroup>(url, query: query, apiVersion: apiVersion);
  }


  // --------------------------------
  // Actions
  // --------------------------------
  // Instance functions to run actions from this item



}
